#pragma clang diagnostic ignored "-Wmissing-prototypes"

#include <metal_stdlib>
#include <simd/simd.h>
#define STD_DISABLE_VERTEX_NORMAL 1
#define STD_DISABLE_VERTEX_TANGENT 1
#define STD_DISABLE_VERTEX_TEXTURE1 1

#include "std2.metal"
#include "std2_vs.metal"
#include "std2_fs.metal"

using namespace metal;

namespace SNAP_VS {


//SG_REFLECTION_BEGIN(100)
//ubo float UserUniforms 0:35568 {
//float4x4 script_modelMatrix 0
//float4x4 script_viewProjectionMatrix 64
//float3 uniSphereCenter 128
//float3 uniCameraPos 144
//float4 sgColorAndSharpness 160:[]:16
//float3 sgAxis :[]:16
//float3 sgAmbientLight
//}
//SG_REFLECTION_END

#ifndef N_SPHERICAL_GAUSSIANS
#define N_SPHERICAL_GAUSSIANS 12
#endif

struct UserUniforms
{
float4x4 script_modelMatrix;
float4x4 script_viewProjectionMatrix;
float3 uniSphereCenter;
float3 uniCameraPos;
float4 sgColorAndSharpness[N_SPHERICAL_GAUSSIANS];
float3 sgAxis[N_SPHERICAL_GAUSSIANS];
float3 sgAmbientLight;
};

struct main_vert_out
{
int varStereoViewID [[user(locn0)]];
float varClipDistance [[user(locn1)]];
float2 varShadowTex [[user(locn2)]];
float3 varPos [[user(locn3)]];
float3 varNormal [[user(locn4)]];
float4 varTangent [[user(locn5)]];
float4 varPackedTex [[user(locn6)]];
float4 varScreenPos [[user(locn7)]];
float2 varScreenTexturePos [[user(locn8)]];
float varViewSpaceDepth [[user(locn9)]];
float3 varCustomNormal [[user(locn10)]];
float4 gl_Position [[position]];
};

struct main_vert_in
{
float4 position [[attribute(0)]];
float3 normal [[attribute(1)]];
float4 tangent [[attribute(2)]];
float2 texture0 [[attribute(3)]];
float2 texture1 [[attribute(4)]];
float4 boneData [[attribute(5)]];
float3 blendShape0Pos [[attribute(6)]];
float3 blendShape1Pos [[attribute(7)]];
float3 blendShape2Pos [[attribute(8)]];
float3 blendShape3Pos [[attribute(9)]];
float3 blendShape4Pos [[attribute(10)]];
float3 blendShape5Pos [[attribute(11)]];
float3 blendShape0Normal [[attribute(12)]];
float3 blendShape1Normal [[attribute(13)]];
float3 blendShape2Normal [[attribute(14)]];
};

vertex main_vert_out main_vert(main_vert_in in [[stage_in]],constant UserUniforms& userUniformsObj,constant LibraryUniforms& libraryUniformsObj,uint gl_InstanceIndex [[instance_id]])
{
main_vert_out out={};
sc_Vertex_t v=sc_LoadVertexAttributes(libraryUniformsObj,in.position,in.normal,in.tangent,in.texture0,in.texture1);
float4x4 rotationMatrix=float4x4(float4(1.0,0.0,0.0,0.0),float4(0.0,1.0,0.0,0.0),float4(0.0,0.0,1.0,0.0),float4(0.0,0.0,0.0,1.0));
rotationMatrix[0]=float4(0.0,0.0,1.0,0.0);
rotationMatrix[2]=float4(-1.0,0.0,0.0,0.0);
float4 newPosition=rotationMatrix*v.position;
float4x4 scaleMatrix=float4x4(float4(12.0,0.0,0.0,0.0),float4(0.0,12.0,0.0,0.0),float4(0.0,0.0,12.0,0.0),float4(0.0,0.0,0.0,12.0));
scaleMatrix[3].w=1.0;
newPosition=scaleMatrix*newPosition;
float3 normal=normalize(in.position.xyz);
out.varCustomNormal=normalize(((userUniformsObj.script_modelMatrix*rotationMatrix)*float4(normal,0.0)).xyz);
v.position=float4(((v.texture0*1.002)*2.0)-float2(1.0),0.0,1.0);
sc_Vertex_t param=v;
sc_ProcessVertex(param,libraryUniformsObj,out.varClipDistance,out.varStereoViewID,out.gl_Position,in.boneData,in.blendShape0Pos,in.blendShape0Normal,in.blendShape1Pos,in.blendShape1Normal,in.blendShape2Pos,in.blendShape2Normal,in.blendShape3Pos,in.blendShape4Pos,in.blendShape5Pos,in.tangent,out.varPos,out.varNormal,out.varTangent,out.varPackedTex,out.varScreenPos,out.varScreenTexturePos,out.varShadowTex,out.varViewSpaceDepth);
return out;
}

} // VERTEX SHADER


namespace SNAP_FS {


//SG_REFLECTION_BEGIN(100)
//ubo float UserUniforms 0:35568 {
//float4x4 script_modelMatrix 0
//float4x4 script_viewProjectionMatrix 64
//float3 uniSphereCenter 128
//float3 uniCameraPos 144
//float4 sgColorAndSharpness 160:[]:16
//float3 sgAxis :[]:16
//float3 sgAmbientLight
//}
//SG_REFLECTION_END

struct SphericalGaussian
{
float3 Amplitude;
float3 Axis;
float Sharpness;
};

#ifndef N_SPHERICAL_GAUSSIANS
#define N_SPHERICAL_GAUSSIANS 12
#endif

struct UserUniforms
{
float4x4 script_modelMatrix;
float4x4 script_viewProjectionMatrix;
float3 uniSphereCenter;
float3 uniCameraPos;
float4 sgColorAndSharpness[N_SPHERICAL_GAUSSIANS];
float3 sgAxis[N_SPHERICAL_GAUSSIANS];
float3 sgAmbientLight;
};

struct main_frag_out
{
float4 FragColor_0 [[color(0)]];
float4 FragColor_1 [[color(1)]];
float4 FragColor_2 [[color(2)]];
float4 FragColor_3 [[color(3)]];
};

struct main_frag_in
{
float3 varCustomNormal [[user(locn10)]];
};

SphericalGaussian getSphericalGaussian(thread const int& index,constant UserUniforms& userUniformsObj)
{
SphericalGaussian sg;
sg.Axis=userUniformsObj.sgAxis[index];
sg.Sharpness=userUniformsObj.sgColorAndSharpness[index].w;
sg.Amplitude=float3(userUniformsObj.sgColorAndSharpness[index].xyz.z,userUniformsObj.sgColorAndSharpness[index].xyz.y,userUniformsObj.sgColorAndSharpness[index].xyz.x);
return sg;
}

float3 evaluateSphericalGaussian(thread const float3& pos,thread const int& idx,constant UserUniforms& userUniformsObj)
{
int param=idx;
SphericalGaussian sg=getSphericalGaussian(param,userUniformsObj);
return sg.Amplitude*exp(sg.Sharpness*(dot(sg.Axis,pos)-1.0));
}

float4 encodeRGBD(thread const float3& rgb)
{
float maxRGB=fast::max(1.0,fast::max(rgb.x,fast::max(rgb.y,rgb.z)));
float D=1.0/maxRGB;
return float4(rgb*D,D);
}

float4 evaluateGaussians(constant UserUniforms& userUniformsObj,thread float3& varCustomNormal)
{
float3 color=userUniformsObj.sgAmbientLight.zyx;
float3 pos=normalize(varCustomNormal);
int i=0;
for (int snapLoopIndex=0; snapLoopIndex==0; snapLoopIndex+=0)
{
if (i<N_SPHERICAL_GAUSSIANS)
{
float3 param=pos;
int param_1=i;
color+=evaluateSphericalGaussian(param,param_1,userUniformsObj);
i++;
continue;
}
else
{
break;
}
}
float3 param_2=color;
return encodeRGBD(param_2);
}

fragment main_frag_out main_frag(main_frag_in in [[stage_in]],constant UserUniforms& userUniformsObj,float4 gl_FragCoord [[position]])
{
main_frag_out out={};
float4 FragColor[4]={};
float4 param=evaluateGaussians(userUniformsObj,in.varCustomNormal);
sc_writeFragData0(param,FragColor);
out.FragColor_0=FragColor[0];
out.FragColor_1=FragColor[1];
out.FragColor_2=FragColor[2];
out.FragColor_3=FragColor[3];
return out;
}

} // FRAGMENT SHADER
