#pragma clang diagnostic ignored "-Wmissing-prototypes"

#include <metal_stdlib>
#include <simd/simd.h>
#define STD_DISABLE_VERTEX_NORMAL 1
#define STD_DISABLE_VERTEX_TANGENT 1
#define STD_DISABLE_VERTEX_TEXTURE1 1

#include "std2_vs.metal"
#include "std2_fs.metal"

using namespace metal;

namespace SNAP_VS {


//SG_REFLECTION_BEGIN(100)
//ubo float UserUniforms 0:35568:160 {
//float4 cameraTextureSize 0
//float4 cameraTextureDims 16
//float4 cameraTextureView 32
//float3x3 cameraTextureTransform 48
//float4 cameraTextureUvMinMax 96
//float4 cameraTextureBorderColor 112
//float4 leftSampleOffsetSize 128
//float4 rightSampleOffsetSize 144
//}
//SG_REFLECTION_END


struct UserUniforms
{
float4 cameraTextureSize;
float4 cameraTextureDims;
float4 cameraTextureView;
float3x3 cameraTextureTransform;
float4 cameraTextureUvMinMax;
float4 cameraTextureBorderColor;
float4 leftSampleOffsetSize;
float4 rightSampleOffsetSize;
};

#ifndef cameraTextureHasSwappedViews
#define cameraTextureHasSwappedViews 0
#elif cameraTextureHasSwappedViews==1
#undef cameraTextureHasSwappedViews
#define cameraTextureHasSwappedViews 1
#endif
#ifndef cameraTextureLayout
#define cameraTextureLayout 0
#endif
#ifndef cameraTextureUV
#define cameraTextureUV 0
#endif
#ifndef SC_USE_UV_TRANSFORM_cameraTexture
#define SC_USE_UV_TRANSFORM_cameraTexture 0
#elif SC_USE_UV_TRANSFORM_cameraTexture==1
#undef SC_USE_UV_TRANSFORM_cameraTexture
#define SC_USE_UV_TRANSFORM_cameraTexture 1
#endif
#ifndef SC_SOFTWARE_WRAP_MODE_U_cameraTexture
#define SC_SOFTWARE_WRAP_MODE_U_cameraTexture -1
#endif
#ifndef SC_SOFTWARE_WRAP_MODE_V_cameraTexture
#define SC_SOFTWARE_WRAP_MODE_V_cameraTexture -1
#endif
#ifndef SC_USE_UV_MIN_MAX_cameraTexture
#define SC_USE_UV_MIN_MAX_cameraTexture 0
#elif SC_USE_UV_MIN_MAX_cameraTexture==1
#undef SC_USE_UV_MIN_MAX_cameraTexture
#define SC_USE_UV_MIN_MAX_cameraTexture 1
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_cameraTexture
#define SC_USE_CLAMP_TO_BORDER_cameraTexture 0
#elif SC_USE_CLAMP_TO_BORDER_cameraTexture==1
#undef SC_USE_CLAMP_TO_BORDER_cameraTexture
#define SC_USE_CLAMP_TO_BORDER_cameraTexture 1
#endif

struct main_vert_out
{
int varStereoViewID [[user(locn0)]];
float varClipDistance [[user(locn1)]];
float2 varShadowTex [[user(locn2)]];
float3 varPos [[user(locn3)]];
float3 varNormal [[user(locn4)]];
float4 varTangent [[user(locn5)]];
float4 varPackedTex [[user(locn6)]];
float4 varScreenPos [[user(locn7)]];
float2 varScreenTexturePos [[user(locn8)]];
float varViewSpaceDepth [[user(locn9)]];
float varLerp [[user(locn10)]];
float2 varLeftSampleCoord [[user(locn11)]];
float2 varRightSampleCoord [[user(locn12)]];
float4 gl_Position [[position]];
};

struct main_vert_in
{
float4 position [[attribute(0)]];
float3 normal [[attribute(1)]];
float4 tangent [[attribute(2)]];
float2 texture0 [[attribute(3)]];
float2 texture1 [[attribute(4)]];
float4 boneData [[attribute(5)]];
float3 blendShape0Pos [[attribute(6)]];
float3 blendShape1Pos [[attribute(7)]];
float3 blendShape2Pos [[attribute(8)]];
float3 blendShape3Pos [[attribute(9)]];
float3 blendShape4Pos [[attribute(10)]];
float3 blendShape5Pos [[attribute(11)]];
float3 blendShape0Normal [[attribute(12)]];
float3 blendShape1Normal [[attribute(13)]];
float3 blendShape2Normal [[attribute(14)]];
};

vertex main_vert_out main_vert(main_vert_in in [[stage_in]],constant UserUniforms& userUniformsObj,constant LibraryUniforms& libraryUniformsObj,uint gl_InstanceIndex [[instance_id]])
{
main_vert_out out={};
sc_Vertex_t v=sc_LoadVertexAttributes(libraryUniformsObj,in.position,in.normal,in.tangent,in.texture0,in.texture1);
float3x3 leftSampleTransform=userUniformsObj.cameraTextureTransform;
leftSampleTransform[0].x*=userUniformsObj.leftSampleOffsetSize.z;
leftSampleTransform[1].y*=userUniformsObj.leftSampleOffsetSize.w;
float3x3 rightSampleTransform=userUniformsObj.cameraTextureTransform;
rightSampleTransform[0].x*=userUniformsObj.rightSampleOffsetSize.z;
rightSampleTransform[1].y*=userUniformsObj.rightSampleOffsetSize.w;
float2 rangedTexCoord=(v.texture0*float2(2.0))-float2(1.0);
float3 leftSampleCoord=float3(rangedTexCoord,1.0)*leftSampleTransform;
float3 rightSampleCoord=float3(rangedTexCoord,1.0)*rightSampleTransform;
out.varLerp=1.0-v.texture0.x;
out.varLeftSampleCoord=((leftSampleCoord.xy+userUniformsObj.leftSampleOffsetSize.xy)*0.5)+float2(0.5);
out.varRightSampleCoord=((rightSampleCoord.xy+userUniformsObj.rightSampleOffsetSize.xy)*0.5)+float2(0.5);
sc_Vertex_t param=v;
sc_ProcessVertex(param,libraryUniformsObj,out.varClipDistance,out.varStereoViewID,out.gl_Position,in.boneData,in.blendShape0Pos,in.blendShape0Normal,in.blendShape1Pos,in.blendShape1Normal,in.blendShape2Pos,in.blendShape2Normal,in.blendShape3Pos,in.blendShape4Pos,in.blendShape5Pos,in.tangent,out.varPos,out.varNormal,out.varTangent,out.varPackedTex,out.varScreenPos,out.varScreenTexturePos,out.varShadowTex,out.varViewSpaceDepth);
return out;
}

} // VERTEX SHADER


namespace SNAP_FS {


//SG_REFLECTION_BEGIN(100)
//sampler sampler cameraTextureSmpSC 0:49025
//texture texture2D cameraTexture 0:52956:0:49025
//ubo float UserUniforms 0:35568:160 {
//float4 cameraTextureSize 0
//float4 cameraTextureDims 16
//float4 cameraTextureView 32
//float3x3 cameraTextureTransform 48
//float4 cameraTextureUvMinMax 96
//float4 cameraTextureBorderColor 112
//float4 leftSampleOffsetSize 128
//float4 rightSampleOffsetSize 144
//}
//SG_REFLECTION_END


struct UserUniforms
{
float4 cameraTextureSize;
float4 cameraTextureDims;
float4 cameraTextureView;
float3x3 cameraTextureTransform;
float4 cameraTextureUvMinMax;
float4 cameraTextureBorderColor;
float4 leftSampleOffsetSize;
float4 rightSampleOffsetSize;
};

#ifndef cameraTextureHasSwappedViews
#define cameraTextureHasSwappedViews 0
#elif cameraTextureHasSwappedViews==1
#undef cameraTextureHasSwappedViews
#define cameraTextureHasSwappedViews 1
#endif
#ifndef cameraTextureLayout
#define cameraTextureLayout 0
#endif
#ifndef SC_SOFTWARE_WRAP_MODE_U_cameraTexture
#define SC_SOFTWARE_WRAP_MODE_U_cameraTexture -1
#endif
#ifndef SC_SOFTWARE_WRAP_MODE_V_cameraTexture
#define SC_SOFTWARE_WRAP_MODE_V_cameraTexture -1
#endif
constant int2 _2153=int2(SC_SOFTWARE_WRAP_MODE_U_cameraTexture,SC_SOFTWARE_WRAP_MODE_V_cameraTexture);
#ifndef SC_USE_UV_MIN_MAX_cameraTexture
#define SC_USE_UV_MIN_MAX_cameraTexture 0
#elif SC_USE_UV_MIN_MAX_cameraTexture==1
#undef SC_USE_UV_MIN_MAX_cameraTexture
#define SC_USE_UV_MIN_MAX_cameraTexture 1
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_cameraTexture
#define SC_USE_CLAMP_TO_BORDER_cameraTexture 0
#elif SC_USE_CLAMP_TO_BORDER_cameraTexture==1
#undef SC_USE_CLAMP_TO_BORDER_cameraTexture
#define SC_USE_CLAMP_TO_BORDER_cameraTexture 1
#endif
constant int2 _2184=int2(SC_SOFTWARE_WRAP_MODE_U_cameraTexture,SC_SOFTWARE_WRAP_MODE_V_cameraTexture);
#ifndef cameraTextureUV
#define cameraTextureUV 0
#endif
#ifndef SC_USE_UV_TRANSFORM_cameraTexture
#define SC_USE_UV_TRANSFORM_cameraTexture 0
#elif SC_USE_UV_TRANSFORM_cameraTexture==1
#undef SC_USE_UV_TRANSFORM_cameraTexture
#define SC_USE_UV_TRANSFORM_cameraTexture 1
#endif

struct main_frag_out
{
float4 FragColor_0 [[color(0)]];
float4 FragColor_1 [[color(1)]];
float4 FragColor_2 [[color(2)]];
float4 FragColor_3 [[color(3)]];
};

struct main_frag_in
{
int varStereoViewID [[user(locn0)]];
float varLerp [[user(locn10)]];
float2 varLeftSampleCoord [[user(locn11)]];
float2 varRightSampleCoord [[user(locn12)]];
};

float2 cameraTextureGetDims2D(constant UserUniforms& userUniformsObj)
{
return userUniformsObj.cameraTextureDims.xy;
}

int cameraTextureGetStereoViewIndex(thread int& varStereoViewID)
{
#if (cameraTextureHasSwappedViews)
{
return 1-sc_GetStereoViewIndex(varStereoViewID);
}
#else
{
return sc_GetStereoViewIndex(varStereoViewID);
}
#endif
}

fragment main_frag_out main_frag(main_frag_in in [[stage_in]],constant UserUniforms& userUniformsObj,texture2d<float> cameraTexture,sampler cameraTextureSmpSC,float4 gl_FragCoord [[position]])
{
main_frag_out out={};
float4 FragColor[4]={};
float3 leftSample=float3(0.0);
float2 param=cameraTextureGetDims2D(userUniformsObj);
int param_1=cameraTextureLayout;
int param_2=cameraTextureGetStereoViewIndex(in.varStereoViewID);
float2 param_3=in.varLeftSampleCoord;
bool param_4=false;
float3x3 param_5=float3x3(float3(1.0,0.0,0.0),float3(0.0,1.0,0.0),float3(0.0,0.0,1.0));
int2 param_6=_2153;
bool param_7=(int(SC_USE_UV_MIN_MAX_cameraTexture)!=0);
float4 param_8=userUniformsObj.cameraTextureUvMinMax;
bool param_9=(int(SC_USE_CLAMP_TO_BORDER_cameraTexture)!=0);
float4 param_10=userUniformsObj.cameraTextureBorderColor;
float param_11=0.0;
float4 _2175=sc_SampleTextureBiasOrLevel(cameraTexture,cameraTextureSmpSC,param,param_1,param_2,param_3,param_4,param_5,param_6,param_7,param_8,param_9,param_10,param_11);
float4 _2142=_2175;
float4 cameraTextureSample=_2175;
leftSample=cameraTextureSample.xyz;
float3 rightSample=float3(0.0);
float2 param_12=cameraTextureGetDims2D(userUniformsObj);
int param_13=cameraTextureLayout;
int param_14=cameraTextureGetStereoViewIndex(in.varStereoViewID);
float2 param_15=in.varRightSampleCoord;
bool param_16=false;
float3x3 param_17=float3x3(float3(1.0,0.0,0.0),float3(0.0,1.0,0.0),float3(0.0,0.0,1.0));
int2 param_18=_2184;
bool param_19=(int(SC_USE_UV_MIN_MAX_cameraTexture)!=0);
float4 param_20=userUniformsObj.cameraTextureUvMinMax;
bool param_21=(int(SC_USE_CLAMP_TO_BORDER_cameraTexture)!=0);
float4 param_22=userUniformsObj.cameraTextureBorderColor;
float param_23=0.0;
float4 _2202=sc_SampleTextureBiasOrLevel(cameraTexture,cameraTextureSmpSC,param_12,param_13,param_14,param_15,param_16,param_17,param_18,param_19,param_20,param_21,param_22,param_23);
float4 _2180=_2202;
float4 cameraTextureSample_1=_2202;
rightSample=cameraTextureSample_1.xyz;
float3 lerpedColor=mix(leftSample,rightSample,float3(in.varLerp));
float4 param_24=float4(lerpedColor,1.0);
sc_writeFragData0(param_24,FragColor);
out.FragColor_0=FragColor[0];
out.FragColor_1=FragColor[1];
out.FragColor_2=FragColor[2];
out.FragColor_3=FragColor[3];
return out;
}

} // FRAGMENT SHADER
