#pragma once
#include <metal_stdlib>
#include <simd/simd.h>
#include "required2.metal"
#include "std2_vs.metal"

using namespace metal;

namespace SNAP_VS {


//SG_REFLECTION_BEGIN(100)
//attribute vec3 positionNext 15
//attribute vec3 positionPrevious 16
//attribute vec4 strandProperties 17
//SG_REFLECTION_END

struct sc_HairVertex
{
float4 position;
float3 normal;
float3 tangent;
float2 texture0;
float4 debugColor;
int strandId;
};

#ifndef sc_HairFallbackMode
#define sc_HairFallbackMode 0
#elif sc_HairFallbackMode==1
#undef sc_HairFallbackMode
#define sc_HairFallbackMode 1
#endif
#ifndef sc_HairDebugMode
#define sc_HairDebugMode 0
#elif sc_HairDebugMode==1
#undef sc_HairDebugMode
#define sc_HairDebugMode 1
#endif

float rand_arg2f(thread const float2& co)
{
return fract(sin(dot(co,float2(12.9898,78.233)))*43758.5);
}

sc_HairVertex sc_GenerateHairVertexInViewspace(thread const sc_Vertex_t& v,constant LibraryUniforms& libraryUniformsObj,thread uint& gl_InstanceIndex,thread float2& texture0,thread texture2d<float> strandDataMapTexture,thread const sampler strandDataMapTextureSmplr,thread float3& positionNext,thread float3& positionPrevious,thread float4& strandProperties)
{
sc_HairVertex hv;
#if (!sc_HairFallbackMode)
{
float instanceId=float(sc_GetLocalInstanceID(gl_InstanceIndex));
float strandInstanceCount=(1.0+libraryUniformsObj.clumpInstanceCount)+libraryUniformsObj.hairstyleInstanceCount;
float instanceCount=libraryUniformsObj.strandDataMapTextureSize.y*strandInstanceCount;
float basicStrandIndex=floor((instanceId/strandInstanceCount)+0.0001);
float basicStrandInstanceId=basicStrandIndex*strandInstanceCount;
float relativeInstanceId=instanceId-basicStrandInstanceId;
float _2416=relativeInstanceId;
bool _2417=_2416>0.0;
bool _2424;
if (_2417)
{
_2424=relativeInstanceId<=libraryUniformsObj.clumpInstanceCount;
}
else
{
_2424=_2417;
}
bool isSingleInterpolationStrand=_2424;
bool isMultiInterpolationStrand=(relativeInstanceId>libraryUniformsObj.clumpInstanceCount)&&(relativeInstanceId<strandInstanceCount);
bool isBasicStrand=(!isSingleInterpolationStrand)&&(!isMultiInterpolationStrand);
float2 uvAbsoluteMax=float2(libraryUniformsObj.strandDataMapTextureSize.x-1.0,libraryUniformsObj.strandDataMapTextureSize.y-1.0);
float2 uvOffsetX=float2(1.0/uvAbsoluteMax.x,0.0);
float2 randomTerms=float2(instanceId/instanceCount,(instanceCount-instanceId)/instanceCount);
float3 randomOffset=float3(0.0);
float2 param=randomTerms.xx;
randomOffset.x=(rand_arg2f(param)*2.0)-1.0;
float2 param_1=float2(randomOffset.x,randomTerms.x);
randomOffset.y=(rand_arg2f(param_1)*2.0)-1.0;
float2 param_2=float2(randomOffset.x,randomOffset.y);
randomOffset.z=(rand_arg2f(param_2)*2.0)-1.0;
float2 uv=float2(v.position.y/uvAbsoluteMax.x,basicStrandIndex/uvAbsoluteMax.y);
bool pointIsNotLast=v.position.y<uvAbsoluteMax.x;
bool pointIsNotFirst=v.position.y>0.0;
float3 pointWorldPosition=strandDataMapTexture.sample(strandDataMapTextureSmplr,uv,level(0.0)).xyz;
float3 _2521;
if (pointIsNotLast)
{
_2521=strandDataMapTexture.sample(strandDataMapTextureSmplr,(uv+uvOffsetX),level(0.0)).xyz;
}
else
{
_2521=pointWorldPosition;
}
float3 pointWorldPositionNext=_2521;
float3 _2535;
if (pointIsNotFirst)
{
_2535=strandDataMapTexture.sample(strandDataMapTextureSmplr,(uv-uvOffsetX),level(0.0)).xyz;
}
else
{
_2535=pointWorldPosition;
}
float3 pointWorldPositionPrevious=_2535;
float3 strandPointOffset=float3(0.0);
float3 strandPointOffsetNext=float3(0.0);
float3 strandPointOffsetPrevious=float3(0.0);
float2 neighbourIndices=float2(-1.0);
if (isMultiInterpolationStrand)
{
neighbourIndices.x=strandDataMapTexture.sample(strandDataMapTextureSmplr,float2(0.0,uv.y),level(0.0)).w;
neighbourIndices.y=strandDataMapTexture.sample(strandDataMapTextureSmplr,float2(uvOffsetX.x,uv.y),level(0.0)).w;
float _2573=neighbourIndices.x;
bool _2574=_2573<0.0;
bool _2580;
if (_2574)
{
_2580=neighbourIndices.y<0.0;
}
else
{
_2580=_2574;
}
if (_2580)
{
isSingleInterpolationStrand=true;
isMultiInterpolationStrand=false;
}
}
if (isMultiInterpolationStrand)
{
float3 pointWorldPositionInterpolated=pointWorldPosition;
float3 pointWorldPositionInterpolatedNext=pointWorldPositionNext;
float3 pointWorldPositionInterpolatedPrevious=pointWorldPositionPrevious;
if (neighbourIndices.x>=0.0)
{
float2 uv1=float2(uv.x,neighbourIndices.x/uvAbsoluteMax.y);
float3 pointWorldPosition1=strandDataMapTexture.sample(strandDataMapTextureSmplr,uv1,level(0.0)).xyz;
float2 param_3=randomTerms;
float lerpTerm1=rand_arg2f(param_3);
pointWorldPositionInterpolated=mix(pointWorldPositionInterpolated,pointWorldPosition1,float3(lerpTerm1));
if (pointIsNotLast)
{
float3 pointWorldPositionNext1=strandDataMapTexture.sample(strandDataMapTextureSmplr,(uv1+uvOffsetX),level(0.0)).xyz;
pointWorldPositionInterpolatedNext=mix(pointWorldPositionInterpolatedNext,pointWorldPositionNext1,float3(lerpTerm1));
}
if (pointIsNotFirst)
{
float3 pointWorldPositionPrevious1=strandDataMapTexture.sample(strandDataMapTextureSmplr,(uv1-uvOffsetX),level(0.0)).xyz;
pointWorldPositionInterpolatedPrevious=mix(pointWorldPositionInterpolatedPrevious,pointWorldPositionPrevious1,float3(lerpTerm1));
}
}
if (neighbourIndices.y>=0.0)
{
float2 uv2=float2(uv.x,neighbourIndices.y/uvAbsoluteMax.y);
float3 pointWorldPosition2=strandDataMapTexture.sample(strandDataMapTextureSmplr,uv2,level(0.0)).xyz;
float2 param_4=randomTerms.xx;
float lerpTerm2=rand_arg2f(param_4);
pointWorldPositionInterpolated=mix(pointWorldPositionInterpolated,pointWorldPosition2,float3(lerpTerm2));
if (pointIsNotLast)
{
float3 pointWorldPositionNext2=strandDataMapTexture.sample(strandDataMapTextureSmplr,(uv2+uvOffsetX),level(0.0)).xyz;
pointWorldPositionInterpolatedNext=mix(pointWorldPositionInterpolatedNext,pointWorldPositionNext2,float3(lerpTerm2));
}
if (pointIsNotFirst)
{
float3 pointWorldPositionPrevious2=strandDataMapTexture.sample(strandDataMapTextureSmplr,(uv2-uvOffsetX),level(0.0)).xyz;
pointWorldPositionInterpolatedPrevious=mix(pointWorldPositionInterpolatedPrevious,pointWorldPositionPrevious2,float3(lerpTerm2));
}
}
float3 noiseOffset=randomOffset*libraryUniformsObj.hairstyleNoise;
strandPointOffset=pointWorldPositionInterpolated-pointWorldPosition;
strandPointOffset+=noiseOffset;
strandPointOffsetNext=pointWorldPositionInterpolatedNext-pointWorldPositionNext;
strandPointOffsetNext+=noiseOffset;
strandPointOffsetPrevious=pointWorldPositionInterpolatedPrevious-pointWorldPositionPrevious;
strandPointOffsetPrevious+=noiseOffset;
}
else
{
if (isSingleInterpolationStrand)
{
float _2744;
if (libraryUniformsObj.clumpTipScale<0.0)
{
_2744=libraryUniformsObj.clumpRadius*libraryUniformsObj.clumpTipScale;
}
else
{
_2744=libraryUniformsObj.clumpTipScale;
}
float clumpTipScaleTerm=_2744;
strandPointOffset=randomOffset*(libraryUniformsObj.clumpRadius+(uv.x*clumpTipScaleTerm));
strandPointOffsetNext=randomOffset*(libraryUniformsObj.clumpRadius+((uv.x+uvOffsetX.x)*clumpTipScaleTerm));
strandPointOffsetPrevious=randomOffset*(libraryUniformsObj.clumpRadius+((uv.x-uvOffsetX.x)*clumpTipScaleTerm));
}
}
pointWorldPosition+=strandPointOffset;
float4 pointViewPosition=libraryUniformsObj.sc_ViewMatrixArray[sc_GetStereoViewIndex()]*float4(pointWorldPosition,1.0);
float3 smoothedTangent=float3(0.0);
if (pointIsNotLast)
{
float4 pointViewPositionNext=libraryUniformsObj.sc_ViewMatrixArray[sc_GetStereoViewIndex()]*float4(pointWorldPositionNext+strandPointOffsetNext,1.0);
smoothedTangent=normalize(pointViewPositionNext.xyz-pointViewPosition.xyz);
}
if (pointIsNotFirst)
{
float4 pointViewPositionPrevious=libraryUniformsObj.sc_ViewMatrixArray[sc_GetStereoViewIndex()]*float4(pointWorldPositionPrevious+strandPointOffsetPrevious,1.0);
smoothedTangent+=normalize(pointViewPosition.xyz-pointViewPositionPrevious.xyz);
}
float _2849=v.position.y;
bool _2850=_2849>0.0;
bool _2858;
if (_2850)
{
_2858=v.position.y<uvAbsoluteMax.x;
}
else
{
_2858=_2850;
}
if (_2858)
{
smoothedTangent*=0.5;
}
hv.tangent=smoothedTangent;
float3 smoothedBinormal=float3(smoothedTangent.y,-smoothedTangent.x,smoothedTangent.z);
hv.normal=normalize(cross(smoothedTangent,smoothedBinormal));
float2 _2900=pointViewPosition.xy+(((smoothedBinormal.xy*v.position.x)*libraryUniformsObj.strandWidth)*(1.0-(uv.x*libraryUniformsObj.strandTaper)));
pointViewPosition=float4(_2900.x,_2900.y,pointViewPosition.z,pointViewPosition.w);
hv.position=pointViewPosition;
hv.texture0=float2(((-v.position.x)*0.5)+0.5,1.0-uv.x);
hv.strandId=sc_GetLocalInstanceID(gl_InstanceIndex);
hv.debugColor=float4(0.0);
#if (sc_HairDebugMode)
{
if (isBasicStrand)
{
hv.debugColor=float4(0.0,1.0,0.0,1.0);
}
else
{
if (isSingleInterpolationStrand)
{
hv.debugColor=float4(0.0,0.0,1.0,1.0);
}
else
{
if (isMultiInterpolationStrand)
{
hv.debugColor=float4(1.0,0.0,0.0,1.0);
}
}
}
}
#endif
}
#else
{
float4 pointViewPosition_1=libraryUniformsObj.sc_ModelViewMatrixArray[sc_GetStereoViewIndex()]*float4(v.position.x,v.position.y,v.position.z,1.0);
float4 pointViewPositionNext_1=libraryUniformsObj.sc_ModelViewMatrixArray[sc_GetStereoViewIndex()]*float4(positionNext.x,positionNext.y,positionNext.z,1.0);
float4 pointViewPositionPrevious_1=libraryUniformsObj.sc_ModelViewMatrixArray[sc_GetStereoViewIndex()]*float4(positionPrevious.x,positionPrevious.y,positionPrevious.z,1.0);
float3 smoothedTangent_1=float3(0.0);
float strandPointNormalizedDistance=strandProperties.y;
if (strandPointNormalizedDistance<1.0)
{
smoothedTangent_1+=normalize(pointViewPositionNext_1.xyz-pointViewPosition_1.xyz);
}
if (strandPointNormalizedDistance>0.0)
{
smoothedTangent_1+=normalize(pointViewPosition_1.xyz-pointViewPositionPrevious_1.xyz);
}
if ((strandPointNormalizedDistance>0.0)&&(strandPointNormalizedDistance<1.0))
{
smoothedTangent_1*=0.5;
}
hv.tangent=smoothedTangent_1;
float binormalSign=strandProperties.x;
float2 smoothedBinormal_1=float2(smoothedTangent_1.y,-smoothedTangent_1.x);
float2 _3044=pointViewPosition_1.xy+(((smoothedBinormal_1*binormalSign)*libraryUniformsObj.strandWidth)*(1.0-((1.0-texture0.y)*libraryUniformsObj.strandTaper)));
pointViewPosition_1=float4(_3044.x,_3044.y,pointViewPosition_1.z,pointViewPosition_1.w);
hv.position=pointViewPosition_1;
hv.texture0=v.texture0;
hv.strandId=int(strandProperties.w);
#if (sc_HairDebugMode)
{
float strandType=strandProperties.z;
hv.debugColor=float4(float(strandType==2.0),float(strandType==0.0),float(strandType==1.0),1.0);
}
#else
{
hv.debugColor=float4(0.0);
}
#endif
}
#endif
return hv;
}

} // VERTEX SHADER


namespace SNAP_FS {


//SG_REFLECTION_BEGIN(100)
//SG_REFLECTION_END

float rand_arg2f(thread const float2& co)
{
return fract(sin(dot(co,float2(12.9898,78.233)))*43758.5);
}

} // FRAGMENT SHADER

