#pragma once
#include <metal_stdlib>
#include <simd/simd.h>
    #ifndef sc_TextureRenderingLayout_Regular
        #define sc_TextureRenderingLayout_Regular 0
        #define sc_TextureRenderingLayout_StereoInstancedClipped 1
        #define sc_TextureRenderingLayout_StereoMultiview 2
    #endif

#define depthToGlobal depthScreenToViewSpace

#define depthToLocal depthViewToScreenSpace

    #ifndef quantizeUV
        #define quantizeUV sc_QuantizeUV
        #define sc_platformUVFlip sc_PlatformFlipUV
    #endif

    #ifndef sc_SampleTexture
        #define sc_SampleTexture sc_SampleTextureBiasOrLevel
    #endif

    #ifndef sc_texture2DLod
        #define sc_texture2DLod sc_InternalTextureLevel
        #define sc_textureLod sc_InternalTextureLevel
        #define sc_textureBias sc_InternalTextureBiasOrLevel
        #define sc_texture sc_InternalTexture
    #endif

#include "required2.metal"
#include "std2_stereo_view_index.metal"
#include "std2_texture_sub.metal"

using namespace metal;

namespace SNAP_VS {


//SG_REFLECTION_BEGIN(100)
//SG_REFLECTION_END


float4 sc_InternalTextureLevelCombined(thread const texture2d<float> combinedSmp,thread const sampler combinedSmpSmplr,thread const float2& texSize,thread const float2& uv,thread const float& level_)
{
return combinedSmp.sample(combinedSmpSmplr,uv,level(level_));
}

float4 sc_InternalTextureLevel(thread const texture2d<float> tex,thread const sampler smp,thread const float2& texSize,thread const float2& uv,thread const float& level_)
{
return tex.sample(smp,uv,level(level_));
}

float4 sc_InternalTextureBiasOrLevel(thread const texture2d<float> tex,thread const sampler smp,thread const float2& texSize,thread const float2& uv,thread const float& biasOrLevel)
{
float2 param=texSize;
float2 param_1=uv;
float param_2=biasOrLevel;
return sc_InternalTextureLevel(tex,smp,param,param_1,param_2);
}

float4 sc_InternalTextureLevel(thread const texture2d<float> tex,thread const sampler smp,thread const float2& uv,thread const float& level_)
{
#if (sc_CanUseTextureLod)
{
return tex.sample(smp,uv,level(level_));
}
#else
{
return float4(0.0);
}
#endif
}

float4 sc_InternalTextureLevel(thread const texture2d_array<float> tex,thread const sampler smp,thread const float3& uv,thread const float& level_)
{
#if (sc_CanUseTextureLod)
{
return tex.sample(smp,uv.xy,uint(round(uv.z)),level(level_));
}
#else
{
return float4(0.0);
}
#endif
}

float4 sc_InternalTextureLevel(thread const texture3d<float> tex,thread const sampler smp,thread const float3& uv,thread const float& level_)
{
#if (sc_CanUseTextureLod)
{
return tex.sample(smp,uv,level(level_));
}
#else
{
return float4(0.0);
}
#endif
}

float4 sc_InternalTextureLevel(thread const texturecube<float> tex,thread const sampler smp,thread const float3& uv,thread const float& level_)
{
#if (sc_CanUseTextureLod)
{
return tex.sample(smp,uv,level(level_));
}
#else
{
return float4(0.0);
}
#endif
}

float4 sc_InternalTextureBiasOrLevel(thread const texture2d<float> tex,thread const sampler smp,thread const float2& uv,thread const float& biasOrLevel)
{
float2 param=uv;
float param_1=biasOrLevel;
return sc_InternalTextureLevel(tex,smp,param,param_1);
}

float4 sc_InternalTextureBiasOrLevel(thread const texture2d_array<float> tex,thread const sampler smp,thread const float3& uv,thread const float& biasOrLevel)
{
float3 param=uv;
float param_1=biasOrLevel;
return sc_InternalTextureLevel(tex,smp,param,param_1);
}

float4 sc_InternalTextureBiasOrLevel(thread const texture3d<float> tex,thread const sampler smp,thread const float3& uv,thread const float& biasOrLevel)
{
float3 param=uv;
float param_1=biasOrLevel;
return sc_InternalTextureLevel(tex,smp,param,param_1);
}

float4 sc_InternalTextureBiasOrLevel(thread const texturecube<float> tex,thread const sampler smp,thread const float3& uv,thread const float& biasOrLevel)
{
float3 param=uv;
float param_1=biasOrLevel;
return sc_InternalTextureLevel(tex,smp,param,param_1);
}

float2 sc_SamplingCoordsGlobalToView(thread float3& uvi,thread const int& renderingLayout,thread const int& viewIndex)
{
if (renderingLayout==1)
{
uvi.y=((2.0*uvi.y)+float(viewIndex))-1.0;
}
return uvi.xy;
}

float2 sc_ScreenCoordsGlobalToView(thread const float2& uv)
{
#if (sc_StereoRenderingMode==1)
{
float3 uvi=float3(uv,0.0);
int instancedClippedLayout=1;
int viewIndex=sc_GetStereoViewIndex();
float3 param=float3(uv,0.0);
int param_1=instancedClippedLayout;
int param_2=viewIndex;
float2 _435=sc_SamplingCoordsGlobalToView(param,param_1,param_2);
return _435;
}
#else
{
return uv;
}
#endif
}

float3 sc_SamplingCoordsViewToGlobal(thread const float2& uv,thread const int& renderingLayout,thread const int& viewIndex)
{
if (renderingLayout==0)
{
return float3(uv,0.0);
}
else
{
if (renderingLayout==1)
{
return float3(uv.x,(uv.y*0.5)+(0.5-(float(viewIndex)*0.5)),0.0);
}
else
{
return float3(uv,float(viewIndex));
}
}
}

float4 sc_SampleViewLevel(thread const texture2d<float> tex,thread const sampler smp,thread const float2& texSize,thread const float2& uv,thread const int& renderingLayout,thread const int& viewIndex,thread const float& level)
{
float2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
float3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
float2 param_3=texSize;
float2 param_4=uvi.xy;
float param_5=level;
return sc_InternalTextureLevel(tex,smp,param_3,param_4,param_5);
}

float4 sc_SampleView(thread const texture2d<float> tex,thread const sampler smp,thread const float2& texSize,thread const float2& uv,thread const int& renderingLayout,thread const int& viewIndex,thread const float& bias0)
{
float2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
float3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
float2 param_3=texSize;
float2 param_4=uvi.xy;
float param_5=bias0;
return sc_InternalTextureBiasOrLevel(tex,smp,param_3,param_4,param_5);
}

float4 sc_SampleViewLevel(thread const texture2d_array<float> tex,thread const sampler smp,thread const float2& texSize,thread const float2& uv,thread const int& renderingLayout,thread const int& viewIndex,thread const float& level)
{
float2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
float3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
float3 param_3=uvi;
float param_4=level;
return sc_InternalTextureLevel(tex,smp,param_3,param_4);
}

float4 sc_SampleView(thread const texture2d_array<float> tex,thread const sampler smp,thread const float2& texSize,thread const float2& uv,thread const int& renderingLayout,thread const int& viewIndex,thread const float& bias0)
{
float2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
float3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
float3 param_3=uvi;
float param_4=bias0;
return sc_InternalTextureBiasOrLevel(tex,smp,param_3,param_4);
}

void sc_ClampUV(thread float& value,thread const float& minValue,thread const float& maxValue,thread const bool& useClampToBorder,thread float& clampToBorderFactor)
{
float clampedValue=fast::clamp(value,minValue,maxValue);
float factor=step(abs(value-clampedValue),1e-05);
clampToBorderFactor*=(factor+((1.0-float(useClampToBorder))*(1.0-factor)));
value=clampedValue;
}

float2 sc_PlatformFlipUV(thread const float2& uv)
{
return float2(uv.x,1.0-uv.y);
}

void sc_SoftwareWrap(thread float& uv,thread const int& softwareWrapMode,thread const bool& useClampToBorder,thread float& clampToBorderFactor)
{
if ((softwareWrapMode==0)||(softwareWrapMode==3))
{
float param=uv;
float param_1=0.0;
float param_2=1.0;
bool param_3=useClampToBorder;
float param_4=clampToBorderFactor;
sc_ClampUV(param,param_1,param_2,param_3,param_4);
uv=param;
clampToBorderFactor=param_4;
}
else
{
if (softwareWrapMode==1)
{
uv=fract(uv);
}
else
{
if (softwareWrapMode==2)
{
float uvFract=fract(uv);
float uvInt=uv-uvFract;
float uvOdd=step(0.25,fract(uvInt*0.5));
uv=mix(uvFract,1.0-uvFract,fast::clamp(uvOdd,0.0,1.0));
}
}
}
}

float2 sc_QuantizeUV(thread const float2& uv)
{
return uv;
}

float4 sc_SampleTextureBiasOrLevel(thread const texture2d<float> texture_,thread const sampler sampler_,thread const float2& samplerDims,thread const int& renderingLayout,thread const int& viewIndex,thread const float2& uv,thread const bool& useUvTransform,thread const float3x3& uvTransform,thread const int2& softwareWrapModes,thread const bool& useUvMinMax,thread const float4& uvMinMax,thread const bool& useClampToBorder,thread const float4& borderColor,thread const float& biasOrLevel)
{
float2 _uv=uv;
float clampToBorderFactor=1.0;
bool useClampToBorderForWrap=useClampToBorder&&(!useUvMinMax);
float param=_uv.x;
int param_1=softwareWrapModes.x;
bool param_2=useClampToBorderForWrap;
float param_3=clampToBorderFactor;
sc_SoftwareWrap(param,param_1,param_2,param_3);
_uv.x=param;
clampToBorderFactor=param_3;
float param_4=_uv.y;
int param_5=softwareWrapModes.y;
bool param_6=useClampToBorderForWrap;
float param_7=clampToBorderFactor;
sc_SoftwareWrap(param_4,param_5,param_6,param_7);
_uv.y=param_4;
clampToBorderFactor=param_7;
if (useUvMinMax)
{
float param_8=_uv.x;
float param_9=uvMinMax.x;
float param_10=uvMinMax.z;
bool param_11=useClampToBorder;
float param_12=clampToBorderFactor;
sc_ClampUV(param_8,param_9,param_10,param_11,param_12);
_uv.x=param_8;
clampToBorderFactor=param_12;
float param_13=_uv.y;
float param_14=uvMinMax.y;
float param_15=uvMinMax.w;
bool param_16=useClampToBorder;
float param_17=clampToBorderFactor;
sc_ClampUV(param_13,param_14,param_15,param_16,param_17);
_uv.y=param_13;
clampToBorderFactor=param_17;
}
if (useUvTransform)
{
_uv=float2((uvTransform*float3(_uv,1.0)).xy);
}
float2 param_18=_uv;
_uv=sc_QuantizeUV(param_18);
float2 param_19=_uv;
_uv=sc_PlatformFlipUV(param_19);
float2 param_20=samplerDims;
float2 param_21=_uv;
int param_22=renderingLayout;
int param_23=viewIndex;
float param_24=biasOrLevel;
float4 tex=sc_SampleView(texture_,sampler_,param_20,param_21,param_22,param_23,param_24);
if (useClampToBorder)
{
tex=mix(borderColor,tex,float4(clampToBorderFactor));
}
return tex;
}

float4 sc_SampleTextureBiasOrLevel(thread const texture2d_array<float> texture_,thread const sampler sampler_,thread const float2& samplerDims,thread const int& renderingLayout,thread const int& viewIndex,thread const float2& uv,thread const bool& useUvTransform,thread const float3x3& uvTransform,thread const int2& softwareWrapModes,thread const bool& useUvMinMax,thread const float4& uvMinMax,thread const bool& useClampToBorder,thread const float4& borderColor,thread const float& biasOrLevel)
{
float2 _uv=uv;
float clampToBorderFactor=1.0;
bool useClampToBorderForWrap=useClampToBorder&&(!useUvMinMax);
float param=_uv.x;
int param_1=softwareWrapModes.x;
bool param_2=useClampToBorderForWrap;
float param_3=clampToBorderFactor;
sc_SoftwareWrap(param,param_1,param_2,param_3);
_uv.x=param;
clampToBorderFactor=param_3;
float param_4=_uv.y;
int param_5=softwareWrapModes.y;
bool param_6=useClampToBorderForWrap;
float param_7=clampToBorderFactor;
sc_SoftwareWrap(param_4,param_5,param_6,param_7);
_uv.y=param_4;
clampToBorderFactor=param_7;
if (useUvMinMax)
{
float param_8=_uv.x;
float param_9=uvMinMax.x;
float param_10=uvMinMax.z;
bool param_11=useClampToBorder;
float param_12=clampToBorderFactor;
sc_ClampUV(param_8,param_9,param_10,param_11,param_12);
_uv.x=param_8;
clampToBorderFactor=param_12;
float param_13=_uv.y;
float param_14=uvMinMax.y;
float param_15=uvMinMax.w;
bool param_16=useClampToBorder;
float param_17=clampToBorderFactor;
sc_ClampUV(param_13,param_14,param_15,param_16,param_17);
_uv.y=param_13;
clampToBorderFactor=param_17;
}
if (useUvTransform)
{
_uv=float2((uvTransform*float3(_uv,1.0)).xy);
}
float2 param_18=_uv;
_uv=sc_QuantizeUV(param_18);
float2 param_19=_uv;
_uv=sc_PlatformFlipUV(param_19);
float2 param_20=samplerDims;
float2 param_21=_uv;
int param_22=renderingLayout;
int param_23=viewIndex;
float param_24=biasOrLevel;
float4 tex=sc_SampleView(texture_,sampler_,param_20,param_21,param_22,param_23,param_24);
if (useClampToBorder)
{
tex=mix(borderColor,tex,float4(clampToBorderFactor));
}
return tex;
}

float4 sc_SampleTextureLevel(thread const texture2d<float> texture_,thread const sampler sampler_,thread const float2& samplerDims,thread const int& renderingLayout,thread const int& viewIndex,thread const float2& uv,thread const bool& useUvTransform,thread const float3x3& uvTransform,thread const int2& softwareWrapModes,thread const bool& useUvMinMax,thread const float4& uvMinMax,thread const bool& useClampToBorder,thread const float4& borderColor,thread const float& level)
{
float2 _uv=uv;
float clampToBorderFactor=1.0;
bool useClampToBorderForWrap=useClampToBorder&&(!useUvMinMax);
float param=_uv.x;
int param_1=softwareWrapModes.x;
bool param_2=useClampToBorderForWrap;
float param_3=clampToBorderFactor;
sc_SoftwareWrap(param,param_1,param_2,param_3);
_uv.x=param;
clampToBorderFactor=param_3;
float param_4=_uv.y;
int param_5=softwareWrapModes.y;
bool param_6=useClampToBorderForWrap;
float param_7=clampToBorderFactor;
sc_SoftwareWrap(param_4,param_5,param_6,param_7);
_uv.y=param_4;
clampToBorderFactor=param_7;
if (useUvMinMax)
{
float param_8=_uv.x;
float param_9=uvMinMax.x;
float param_10=uvMinMax.z;
bool param_11=useClampToBorder;
float param_12=clampToBorderFactor;
sc_ClampUV(param_8,param_9,param_10,param_11,param_12);
_uv.x=param_8;
clampToBorderFactor=param_12;
float param_13=_uv.y;
float param_14=uvMinMax.y;
float param_15=uvMinMax.w;
bool param_16=useClampToBorder;
float param_17=clampToBorderFactor;
sc_ClampUV(param_13,param_14,param_15,param_16,param_17);
_uv.y=param_13;
clampToBorderFactor=param_17;
}
if (useUvTransform)
{
_uv=float2((uvTransform*float3(_uv,1.0)).xy);
}
float2 param_18=_uv;
_uv=sc_QuantizeUV(param_18);
float2 param_19=_uv;
_uv=sc_PlatformFlipUV(param_19);
float2 param_20=samplerDims;
float2 param_21=_uv;
int param_22=renderingLayout;
int param_23=viewIndex;
float param_24=level;
float4 tex=sc_SampleViewLevel(texture_,sampler_,param_20,param_21,param_22,param_23,param_24);
if (useClampToBorder)
{
tex=mix(borderColor,tex,float4(clampToBorderFactor));
}
return tex;
}

float4 sc_SampleTextureLevel(thread const texture2d_array<float> texture_,thread const sampler sampler_,thread const float2& samplerDims,thread const int& renderingLayout,thread const int& viewIndex,thread const float2& uv,thread const bool& useUvTransform,thread const float3x3& uvTransform,thread const int2& softwareWrapModes,thread const bool& useUvMinMax,thread const float4& uvMinMax,thread const bool& useClampToBorder,thread const float4& borderColor,thread const float& level)
{
float2 _uv=uv;
float clampToBorderFactor=1.0;
bool useClampToBorderForWrap=useClampToBorder&&(!useUvMinMax);
float param=_uv.x;
int param_1=softwareWrapModes.x;
bool param_2=useClampToBorderForWrap;
float param_3=clampToBorderFactor;
sc_SoftwareWrap(param,param_1,param_2,param_3);
_uv.x=param;
clampToBorderFactor=param_3;
float param_4=_uv.y;
int param_5=softwareWrapModes.y;
bool param_6=useClampToBorderForWrap;
float param_7=clampToBorderFactor;
sc_SoftwareWrap(param_4,param_5,param_6,param_7);
_uv.y=param_4;
clampToBorderFactor=param_7;
if (useUvMinMax)
{
float param_8=_uv.x;
float param_9=uvMinMax.x;
float param_10=uvMinMax.z;
bool param_11=useClampToBorder;
float param_12=clampToBorderFactor;
sc_ClampUV(param_8,param_9,param_10,param_11,param_12);
_uv.x=param_8;
clampToBorderFactor=param_12;
float param_13=_uv.y;
float param_14=uvMinMax.y;
float param_15=uvMinMax.w;
bool param_16=useClampToBorder;
float param_17=clampToBorderFactor;
sc_ClampUV(param_13,param_14,param_15,param_16,param_17);
_uv.y=param_13;
clampToBorderFactor=param_17;
}
if (useUvTransform)
{
_uv=float2((uvTransform*float3(_uv,1.0)).xy);
}
float2 param_18=_uv;
_uv=sc_QuantizeUV(param_18);
float2 param_19=_uv;
_uv=sc_PlatformFlipUV(param_19);
float2 param_20=samplerDims;
float2 param_21=_uv;
int param_22=renderingLayout;
int param_23=viewIndex;
float param_24=level;
float4 tex=sc_SampleViewLevel(texture_,sampler_,param_20,param_21,param_22,param_23,param_24);
if (useClampToBorder)
{
tex=mix(borderColor,tex,float4(clampToBorderFactor));
}
return tex;
}

float4 sampleTextureWithTransform(thread const texture2d_array<float> tex,thread const sampler smp,thread float3& uv,thread const bool& useLOD,thread const float& level,thread const bool& useUvTransform,thread const float3x3& uvTransform)
{
if (useUvTransform)
{
float2 _1099=float2((uvTransform*float3(uv.xy,1.0)).xy);
uv=float3(_1099.x,_1099.y,uv.z);
}
float2 param=uv.xy;
float2 _1105=sc_QuantizeUV(param);
uv=float3(_1105.x,_1105.y,uv.z);
float3 param_1=uv;
float param_2=level;
return sc_InternalTextureLevel(tex,smp,param_1,param_2);
}

float depthScreenToViewSpace(thread float& depth,thread const float2& projectionMatrixTerms)
{
float m22=projectionMatrixTerms.x;
float m32=projectionMatrixTerms.y;
depth=(depth*2.0)-1.0;
return m32/((-depth)-m22);
}

float depthViewToScreenSpace(thread float& depth,thread const float2& projectionMatrixTerms)
{
float m22=projectionMatrixTerms.x;
float m32=projectionMatrixTerms.y;
float _1140;
if (depth!=0.0)
{
_1140=(-m22)-(m32/depth);
}
else
{
_1140=0.0;
}
depth=_1140;
return (depth*0.5)+0.5;
}

float4 sampleTextureWithTransform(thread const texture2d<float> tex,thread const sampler smp,thread float2& uv,thread const bool& useUvTransform,thread const float3x3& uvTransform)
{
if (useUvTransform)
{
uv=float2((uvTransform*float3(uv,1.0)).xy);
}
float2 param=uv;
uv=sc_QuantizeUV(param);
float2 param_1=uv;
float param_2=0.0;
return sc_InternalTextureBiasOrLevel(tex,smp,param_1,param_2);
}

} // VERTEX SHADER


namespace SNAP_FS {


//SG_REFLECTION_BEGIN(100)
//SG_REFLECTION_END


float4 sc_InternalTextureLevelCombined(thread const texture2d<float> combinedSmp,thread const sampler combinedSmpSmplr,thread const float2& texSize,thread const float2& uv,thread const float& level_)
{
return combinedSmp.sample(combinedSmpSmplr,uv,level(level_));
}

float4 sc_InternalTextureLevel(thread const texture2d<float> tex,thread const sampler smp,thread const float2& texSize,thread const float2& uv,thread const float& level_)
{
return tex.sample(smp,uv,level(level_));
}

float4 sc_InternalTextureBiasOrLevel(thread const texture2d<float> tex,thread const sampler smp,thread const float2& texSize,thread const float2& uv,thread const float& biasOrLevel)
{
return tex.sample(smp,uv,bias(biasOrLevel));
}

float4 sc_InternalTextureLevel(thread const texture2d<float> tex,thread const sampler smp,thread const float2& uv,thread const float& level_)
{
#if (sc_CanUseTextureLod)
{
return tex.sample(smp,uv,level(level_));
}
#else
{
return float4(0.0);
}
#endif
}

float4 sc_InternalTextureLevel(thread const texture2d_array<float> tex,thread const sampler smp,thread const float3& uv,thread const float& level_)
{
#if (sc_CanUseTextureLod)
{
return tex.sample(smp,uv.xy,uint(round(uv.z)),level(level_));
}
#else
{
return float4(0.0);
}
#endif
}

float4 sc_InternalTextureLevel(thread const texture3d<float> tex,thread const sampler smp,thread const float3& uv,thread const float& level_)
{
#if (sc_CanUseTextureLod)
{
return tex.sample(smp,uv,level(level_));
}
#else
{
return float4(0.0);
}
#endif
}

float4 sc_InternalTextureLevel(thread const texturecube<float> tex,thread const sampler smp,thread const float3& uv,thread const float& level_)
{
#if (sc_CanUseTextureLod)
{
return tex.sample(smp,uv,level(level_));
}
#else
{
return float4(0.0);
}
#endif
}

float4 sc_InternalTextureBiasOrLevel(thread const texture2d<float> tex,thread const sampler smp,thread const float2& uv,thread const float& biasOrLevel)
{
return tex.sample(smp,uv,bias(biasOrLevel));
}

float4 sc_InternalTextureBiasOrLevel(thread const texture2d_array<float> tex,thread const sampler smp,thread const float3& uv,thread const float& biasOrLevel)
{
return tex.sample(smp,uv.xy,uint(round(uv.z)),bias(biasOrLevel));
}

float4 sc_InternalTextureBiasOrLevel(thread const texture3d<float> tex,thread const sampler smp,thread const float3& uv,thread const float& biasOrLevel)
{
return tex.sample(smp,uv,bias(biasOrLevel));
}

float4 sc_InternalTextureBiasOrLevel(thread const texturecube<float> tex,thread const sampler smp,thread const float3& uv,thread const float& biasOrLevel)
{
return tex.sample(smp,uv,bias(biasOrLevel));
}

float2 sc_SamplingCoordsGlobalToView(thread float3& uvi,thread const int& renderingLayout,thread const int& viewIndex)
{
if (renderingLayout==1)
{
uvi.y=((2.0*uvi.y)+float(viewIndex))-1.0;
}
return uvi.xy;
}

float2 sc_ScreenCoordsGlobalToView(thread const float2& uv,thread int& varStereoViewID)
{
#if (sc_StereoRenderingMode==1)
{
float3 uvi=float3(uv,0.0);
int instancedClippedLayout=1;
int viewIndex=sc_GetStereoViewIndex(varStereoViewID);
float3 param=float3(uv,0.0);
int param_1=instancedClippedLayout;
int param_2=viewIndex;
float2 _440=sc_SamplingCoordsGlobalToView(param,param_1,param_2);
return _440;
}
#else
{
return uv;
}
#endif
}

float3 sc_SamplingCoordsViewToGlobal(thread const float2& uv,thread const int& renderingLayout,thread const int& viewIndex)
{
if (renderingLayout==0)
{
return float3(uv,0.0);
}
else
{
if (renderingLayout==1)
{
return float3(uv.x,(uv.y*0.5)+(0.5-(float(viewIndex)*0.5)),0.0);
}
else
{
return float3(uv,float(viewIndex));
}
}
}

float4 sc_SampleViewLevel(thread const texture2d<float> tex,thread const sampler smp,thread const float2& texSize,thread const float2& uv,thread const int& renderingLayout,thread const int& viewIndex,thread const float& level)
{
float2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
float3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
float2 param_3=texSize;
float2 param_4=uvi.xy;
float param_5=level;
return sc_InternalTextureLevel(tex,smp,param_3,param_4,param_5);
}

float4 sc_SampleView(thread const texture2d<float> tex,thread const sampler smp,thread const float2& texSize,thread const float2& uv,thread const int& renderingLayout,thread const int& viewIndex,thread const float& bias0)
{
float2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
float3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
float2 param_3=texSize;
float2 param_4=uvi.xy;
float param_5=bias0;
return sc_InternalTextureBiasOrLevel(tex,smp,param_3,param_4,param_5);
}

float4 sc_SampleViewLevel(thread const texture2d_array<float> tex,thread const sampler smp,thread const float2& texSize,thread const float2& uv,thread const int& renderingLayout,thread const int& viewIndex,thread const float& level)
{
float2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
float3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
float3 param_3=uvi;
float param_4=level;
return sc_InternalTextureLevel(tex,smp,param_3,param_4);
}

float4 sc_SampleView(thread const texture2d_array<float> tex,thread const sampler smp,thread const float2& texSize,thread const float2& uv,thread const int& renderingLayout,thread const int& viewIndex,thread const float& bias0)
{
float2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
float3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
float3 param_3=uvi;
float param_4=bias0;
return sc_InternalTextureBiasOrLevel(tex,smp,param_3,param_4);
}

void sc_ClampUV(thread float& value,thread const float& minValue,thread const float& maxValue,thread const bool& useClampToBorder,thread float& clampToBorderFactor)
{
float clampedValue=fast::clamp(value,minValue,maxValue);
float factor=step(abs(value-clampedValue),1e-05);
clampToBorderFactor*=(factor+((1.0-float(useClampToBorder))*(1.0-factor)));
value=clampedValue;
}

float2 sc_PlatformFlipUV(thread const float2& uv)
{
return float2(uv.x,1.0-uv.y);
}

void sc_SoftwareWrap(thread float& uv,thread const int& softwareWrapMode,thread const bool& useClampToBorder,thread float& clampToBorderFactor)
{
if ((softwareWrapMode==0)||(softwareWrapMode==3))
{
float param=uv;
float param_1=0.0;
float param_2=1.0;
bool param_3=useClampToBorder;
float param_4=clampToBorderFactor;
sc_ClampUV(param,param_1,param_2,param_3,param_4);
uv=param;
clampToBorderFactor=param_4;
}
else
{
if (softwareWrapMode==1)
{
uv=fract(uv);
}
else
{
if (softwareWrapMode==2)
{
float uvFract=fract(uv);
float uvInt=uv-uvFract;
float uvOdd=step(0.25,fract(uvInt*0.5));
uv=mix(uvFract,1.0-uvFract,fast::clamp(uvOdd,0.0,1.0));
}
}
}
}

float2 sc_QuantizeUV(thread const float2& uv)
{
return uv;
}

float4 sc_SampleTextureBiasOrLevel(thread const texture2d<float> texture_,thread const sampler sampler_,thread const float2& samplerDims,thread const int& renderingLayout,thread const int& viewIndex,thread const float2& uv,thread const bool& useUvTransform,thread const float3x3& uvTransform,thread const int2& softwareWrapModes,thread const bool& useUvMinMax,thread const float4& uvMinMax,thread const bool& useClampToBorder,thread const float4& borderColor,thread const float& biasOrLevel)
{
float2 _uv=uv;
float clampToBorderFactor=1.0;
bool useClampToBorderForWrap=useClampToBorder&&(!useUvMinMax);
float param=_uv.x;
int param_1=softwareWrapModes.x;
bool param_2=useClampToBorderForWrap;
float param_3=clampToBorderFactor;
sc_SoftwareWrap(param,param_1,param_2,param_3);
_uv.x=param;
clampToBorderFactor=param_3;
float param_4=_uv.y;
int param_5=softwareWrapModes.y;
bool param_6=useClampToBorderForWrap;
float param_7=clampToBorderFactor;
sc_SoftwareWrap(param_4,param_5,param_6,param_7);
_uv.y=param_4;
clampToBorderFactor=param_7;
if (useUvMinMax)
{
float param_8=_uv.x;
float param_9=uvMinMax.x;
float param_10=uvMinMax.z;
bool param_11=useClampToBorder;
float param_12=clampToBorderFactor;
sc_ClampUV(param_8,param_9,param_10,param_11,param_12);
_uv.x=param_8;
clampToBorderFactor=param_12;
float param_13=_uv.y;
float param_14=uvMinMax.y;
float param_15=uvMinMax.w;
bool param_16=useClampToBorder;
float param_17=clampToBorderFactor;
sc_ClampUV(param_13,param_14,param_15,param_16,param_17);
_uv.y=param_13;
clampToBorderFactor=param_17;
}
if (useUvTransform)
{
_uv=float2((uvTransform*float3(_uv,1.0)).xy);
}
float2 param_18=_uv;
_uv=sc_QuantizeUV(param_18);
float2 param_19=_uv;
_uv=sc_PlatformFlipUV(param_19);
float2 param_20=samplerDims;
float2 param_21=_uv;
int param_22=renderingLayout;
int param_23=viewIndex;
float param_24=biasOrLevel;
float4 tex=sc_SampleView(texture_,sampler_,param_20,param_21,param_22,param_23,param_24);
if (useClampToBorder)
{
tex=mix(borderColor,tex,float4(clampToBorderFactor));
}
return tex;
}

float4 sc_SampleTextureBiasOrLevel(thread const texture2d_array<float> texture_,thread const sampler sampler_,thread const float2& samplerDims,thread const int& renderingLayout,thread const int& viewIndex,thread const float2& uv,thread const bool& useUvTransform,thread const float3x3& uvTransform,thread const int2& softwareWrapModes,thread const bool& useUvMinMax,thread const float4& uvMinMax,thread const bool& useClampToBorder,thread const float4& borderColor,thread const float& biasOrLevel)
{
float2 _uv=uv;
float clampToBorderFactor=1.0;
bool useClampToBorderForWrap=useClampToBorder&&(!useUvMinMax);
float param=_uv.x;
int param_1=softwareWrapModes.x;
bool param_2=useClampToBorderForWrap;
float param_3=clampToBorderFactor;
sc_SoftwareWrap(param,param_1,param_2,param_3);
_uv.x=param;
clampToBorderFactor=param_3;
float param_4=_uv.y;
int param_5=softwareWrapModes.y;
bool param_6=useClampToBorderForWrap;
float param_7=clampToBorderFactor;
sc_SoftwareWrap(param_4,param_5,param_6,param_7);
_uv.y=param_4;
clampToBorderFactor=param_7;
if (useUvMinMax)
{
float param_8=_uv.x;
float param_9=uvMinMax.x;
float param_10=uvMinMax.z;
bool param_11=useClampToBorder;
float param_12=clampToBorderFactor;
sc_ClampUV(param_8,param_9,param_10,param_11,param_12);
_uv.x=param_8;
clampToBorderFactor=param_12;
float param_13=_uv.y;
float param_14=uvMinMax.y;
float param_15=uvMinMax.w;
bool param_16=useClampToBorder;
float param_17=clampToBorderFactor;
sc_ClampUV(param_13,param_14,param_15,param_16,param_17);
_uv.y=param_13;
clampToBorderFactor=param_17;
}
if (useUvTransform)
{
_uv=float2((uvTransform*float3(_uv,1.0)).xy);
}
float2 param_18=_uv;
_uv=sc_QuantizeUV(param_18);
float2 param_19=_uv;
_uv=sc_PlatformFlipUV(param_19);
float2 param_20=samplerDims;
float2 param_21=_uv;
int param_22=renderingLayout;
int param_23=viewIndex;
float param_24=biasOrLevel;
float4 tex=sc_SampleView(texture_,sampler_,param_20,param_21,param_22,param_23,param_24);
if (useClampToBorder)
{
tex=mix(borderColor,tex,float4(clampToBorderFactor));
}
return tex;
}

float4 sc_SampleTextureLevel(thread const texture2d<float> texture_,thread const sampler sampler_,thread const float2& samplerDims,thread const int& renderingLayout,thread const int& viewIndex,thread const float2& uv,thread const bool& useUvTransform,thread const float3x3& uvTransform,thread const int2& softwareWrapModes,thread const bool& useUvMinMax,thread const float4& uvMinMax,thread const bool& useClampToBorder,thread const float4& borderColor,thread const float& level)
{
float2 _uv=uv;
float clampToBorderFactor=1.0;
bool useClampToBorderForWrap=useClampToBorder&&(!useUvMinMax);
float param=_uv.x;
int param_1=softwareWrapModes.x;
bool param_2=useClampToBorderForWrap;
float param_3=clampToBorderFactor;
sc_SoftwareWrap(param,param_1,param_2,param_3);
_uv.x=param;
clampToBorderFactor=param_3;
float param_4=_uv.y;
int param_5=softwareWrapModes.y;
bool param_6=useClampToBorderForWrap;
float param_7=clampToBorderFactor;
sc_SoftwareWrap(param_4,param_5,param_6,param_7);
_uv.y=param_4;
clampToBorderFactor=param_7;
if (useUvMinMax)
{
float param_8=_uv.x;
float param_9=uvMinMax.x;
float param_10=uvMinMax.z;
bool param_11=useClampToBorder;
float param_12=clampToBorderFactor;
sc_ClampUV(param_8,param_9,param_10,param_11,param_12);
_uv.x=param_8;
clampToBorderFactor=param_12;
float param_13=_uv.y;
float param_14=uvMinMax.y;
float param_15=uvMinMax.w;
bool param_16=useClampToBorder;
float param_17=clampToBorderFactor;
sc_ClampUV(param_13,param_14,param_15,param_16,param_17);
_uv.y=param_13;
clampToBorderFactor=param_17;
}
if (useUvTransform)
{
_uv=float2((uvTransform*float3(_uv,1.0)).xy);
}
float2 param_18=_uv;
_uv=sc_QuantizeUV(param_18);
float2 param_19=_uv;
_uv=sc_PlatformFlipUV(param_19);
float2 param_20=samplerDims;
float2 param_21=_uv;
int param_22=renderingLayout;
int param_23=viewIndex;
float param_24=level;
float4 tex=sc_SampleViewLevel(texture_,sampler_,param_20,param_21,param_22,param_23,param_24);
if (useClampToBorder)
{
tex=mix(borderColor,tex,float4(clampToBorderFactor));
}
return tex;
}

float4 sc_SampleTextureLevel(thread const texture2d_array<float> texture_,thread const sampler sampler_,thread const float2& samplerDims,thread const int& renderingLayout,thread const int& viewIndex,thread const float2& uv,thread const bool& useUvTransform,thread const float3x3& uvTransform,thread const int2& softwareWrapModes,thread const bool& useUvMinMax,thread const float4& uvMinMax,thread const bool& useClampToBorder,thread const float4& borderColor,thread const float& level)
{
float2 _uv=uv;
float clampToBorderFactor=1.0;
bool useClampToBorderForWrap=useClampToBorder&&(!useUvMinMax);
float param=_uv.x;
int param_1=softwareWrapModes.x;
bool param_2=useClampToBorderForWrap;
float param_3=clampToBorderFactor;
sc_SoftwareWrap(param,param_1,param_2,param_3);
_uv.x=param;
clampToBorderFactor=param_3;
float param_4=_uv.y;
int param_5=softwareWrapModes.y;
bool param_6=useClampToBorderForWrap;
float param_7=clampToBorderFactor;
sc_SoftwareWrap(param_4,param_5,param_6,param_7);
_uv.y=param_4;
clampToBorderFactor=param_7;
if (useUvMinMax)
{
float param_8=_uv.x;
float param_9=uvMinMax.x;
float param_10=uvMinMax.z;
bool param_11=useClampToBorder;
float param_12=clampToBorderFactor;
sc_ClampUV(param_8,param_9,param_10,param_11,param_12);
_uv.x=param_8;
clampToBorderFactor=param_12;
float param_13=_uv.y;
float param_14=uvMinMax.y;
float param_15=uvMinMax.w;
bool param_16=useClampToBorder;
float param_17=clampToBorderFactor;
sc_ClampUV(param_13,param_14,param_15,param_16,param_17);
_uv.y=param_13;
clampToBorderFactor=param_17;
}
if (useUvTransform)
{
_uv=float2((uvTransform*float3(_uv,1.0)).xy);
}
float2 param_18=_uv;
_uv=sc_QuantizeUV(param_18);
float2 param_19=_uv;
_uv=sc_PlatformFlipUV(param_19);
float2 param_20=samplerDims;
float2 param_21=_uv;
int param_22=renderingLayout;
int param_23=viewIndex;
float param_24=level;
float4 tex=sc_SampleViewLevel(texture_,sampler_,param_20,param_21,param_22,param_23,param_24);
if (useClampToBorder)
{
tex=mix(borderColor,tex,float4(clampToBorderFactor));
}
return tex;
}

float4 sampleTextureWithTransform(thread const texture2d_array<float> tex,thread const sampler smp,thread float3& uv,thread const bool& useLOD,thread const float& level,thread const bool& useUvTransform,thread const float3x3& uvTransform)
{
if (useUvTransform)
{
float2 _1104=float2((uvTransform*float3(uv.xy,1.0)).xy);
uv=float3(_1104.x,_1104.y,uv.z);
}
float2 param=uv.xy;
float2 _1110=sc_QuantizeUV(param);
uv=float3(_1110.x,_1110.y,uv.z);
float4 _1114;
if (useLOD)
{
float3 param_1=uv;
float param_2=level;
_1114=sc_InternalTextureLevel(tex,smp,param_1,param_2);
}
else
{
float3 param_3=uv;
float param_4=0.0;
_1114=sc_InternalTextureBiasOrLevel(tex,smp,param_3,param_4);
}
return _1114;
}

float depthScreenToViewSpace(thread float& depth,thread const float2& projectionMatrixTerms)
{
float m22=projectionMatrixTerms.x;
float m32=projectionMatrixTerms.y;
depth=(depth*2.0)-1.0;
return m32/((-depth)-m22);
}

float depthViewToScreenSpace(thread float& depth,thread const float2& projectionMatrixTerms)
{
float m22=projectionMatrixTerms.x;
float m32=projectionMatrixTerms.y;
float _1155;
if (depth!=0.0)
{
_1155=(-m22)-(m32/depth);
}
else
{
_1155=0.0;
}
depth=_1155;
return (depth*0.5)+0.5;
}

float4 sampleTextureWithTransform(thread const texture2d<float> tex,thread const sampler smp,thread float2& uv,thread const bool& useUvTransform,thread const float3x3& uvTransform)
{
if (useUvTransform)
{
uv=float2((uvTransform*float3(uv,1.0)).xy);
}
float2 param=uv;
uv=sc_QuantizeUV(param);
float2 param_1=uv;
float param_2=0.0;
return sc_InternalTextureBiasOrLevel(tex,smp,param_1,param_2);
}

} // FRAGMENT SHADER

